package com.auchan.proto.frontend.custom;

import java.util.List;

import inputSchema.Attribut;
import inputSchema.ComplementAttribut;
import inputSchema.Ecran;
import inputSchema.Ihm;
import inputSchema.InputSchemaFactory;
import inputSchema.Metier;
import inputSchema.Popup;

import org.eclipse.core.resources.IMarker;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.EnumerationLiteral;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLPackage;

import com.ibm.xtools.transform.authoring.RuleExtension;

public class Class2MetierRule implements RuleExtension {

	@SuppressWarnings("unchecked")
	public void execute(EObject arg0, EObject arg1) {
		Util.log("Class2MetierRule", arg0);
		Class cl = (Class) arg0;
		Metier metier = (Metier) arg1;
		// Nom
		metier.setNom(cl.getName());
		// Table
		String table = cl.getName().toUpperCase();
		Stereotype st = cl.getAppliedStereotype("ProtoAuchan::metier");
		if (st != null) {
			String s = (String) cl.getValue(st, "table");
			if (s != null & s.length() > 0) table = s;
		}
		metier.setTable(table);
		// Ecran
		Ihm ihm = null;
		Ecran ecran = null;
		st = cl.getAppliedStereotype("ProtoAuchan::ecran");
		if (st != null) {
			ihm = InputSchemaFactory.eINSTANCE.createIhm();
			ecran = InputSchemaFactory.eINSTANCE.createEcran();
			ihm.getEcran().add(ecran);
		}
		// Popup: association with <<popup>> stereotype pointing to a <<metier>> class
		List<EObject> assocs = Util.findElements(cl.getModel(), UMLPackage.eINSTANCE.getAssociation());
		for (EObject curAssoc_o : assocs) {
			Association curAssoc = (Association) curAssoc_o;
			st = curAssoc.getAppliedStereotype("ProtoAuchan::popup");
			if (st != null) {
				Classifier otherEnd = Util.getOtherEnd(curAssoc, cl);
				if (otherEnd != null && otherEnd instanceof Class) {
					st = otherEnd.getAppliedStereotype("ProtoAuchan::metier");
					if (st == null) {
						Util.addMarker(cl.getNamespace(), "La classe <<metier>> '" + cl.getName() + "' a une association <<popup>> avec la classe '" +
								otherEnd.getName() + "', mais cette dernire n'est pas strotype <<metier>> et a t ignore.", 
								IMarker.SEVERITY_ERROR);
					} else {
						if (ihm == null) {
							ihm = InputSchemaFactory.eINSTANCE.createIhm();					
						}
						Popup popup = InputSchemaFactory.eINSTANCE.createPopup();
						ihm.getPopup().add(popup);
						popup.setCibleMetier(otherEnd.getName());						
					}
				}
			}
		}
		if (ihm != null) {
			metier.getIhm().add(ihm);
		}
		
		// Attributs
		for (Object curAtt_o : cl.getOwnedAttributes()) {
			Property curAtt = (Property) curAtt_o;
			// Ne traiter que si le type n'est pas null
			Type type = curAtt.getType();
			if (type != null) {
				Attribut att = InputSchemaFactory.eINSTANCE.createAttribut();
				metier.getAttribut().add(att);
				att.setNom(curAtt.getName());
				att.setType(type.getName());
				String isPopup = "false";
				if (ihm != null) {
					for (Object o : ihm.getPopup()) {
						Popup p = (Popup) o;
						if (type.getName().equals(p.getCibleMetier())) {
							isPopup = "true";
							break;
						}
					}
				}
				att.setIsPopup(isPopup);
				Stereotype ast = curAtt.getAppliedStereotype("ProtoAuchan::options");
				if (ast != null) {
					String s = (String) curAtt.getValue(ast, "column");
					if (s == null || s.equals("")) {
						s = curAtt.getName().toUpperCase();
					}
					att.setColumn(s);
					Integer size = (Integer) curAtt.getValue(ast, "size");
					EnumerationLiteral converter = (EnumerationLiteral) curAtt.getValue(ast, "converter");
					boolean hasToolTip = ((Boolean) curAtt.getValue(ast, "hasToolTip")).booleanValue();
					if (size > 0 || !converter.getName().equals("none") || hasToolTip) {
						if (ecran != null) {
							ComplementAttribut comp = InputSchemaFactory.eINSTANCE.createComplementAttribut();
							ecran.getComplementAttribut().add(comp);
							comp.setNom(curAtt.getName());
							comp.setConverter(converter.getName());
							comp.setSize(size.toString());
							if (hasToolTip)
								comp.setHasToolTip("true");
							else
								comp.setHasToolTip("false");
						}						
					}
				} else {
					att.setColumn(curAtt.getName().toUpperCase());
				}
				ast = curAtt.getAppliedStereotype("ProtoAuchan::id");
				if (ast != null) {
					att.setIsPrimary("true");
				} else {
					att.setIsPrimary("false");
				}
			}
		}

		
	}

}
