package com.auchan.proto.frontend.custom;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;

public class Util {
	public static void log(String from, EObject arg0) {
		System.out.println(from + ": " + objectheader(arg0));
	}
	
	public static void log(String from, Object arg0) {
		if (arg0 instanceof EObject) {
			System.out.println(from + ": " + objectheader((EObject) arg0));
		} else {
			System.out.println(from + ": " + arg0.getClass().getSimpleName());
		}
	}
	
	public static void log(String from, EObject arg0, EObject arg1) {
		System.out.println(from + ": " + objectheader(arg0) + "; " + objectheader(arg1));
	}
	
	private static String objectheader (EObject obj) {
		String s = obj.eClass().getName() + " ";
		if (obj instanceof NamedElement) {
			s += ((NamedElement) obj).getName();
		} else {
			s += " (not a named element)";
		}
		return s;
	}
	
    public static IResource getModelResource(NamedElement elt) {
        Model model = elt.getModel();
        Resource r = model.eResource();
        if (r != null) {
			IWorkspaceRoot wr = ResourcesPlugin.getWorkspace().getRoot();
			String uri = r.getURI().path().replaceAll("%20", " ");
			if (uri.startsWith("/resource/")) {
				String pName = uri.substring("/resource".length() + 1);
				String mName = null;
				int pos = pName.indexOf("/");
				if (pos > 0) {
					mName = pName.substring(pos + 1);
					pName = pName.substring(0, pos);
					IProject prj = wr.getProject(pName);
					if (prj != null) {
						return prj.findMember(mName);
					}
				}
			}
		}
        return null;
    }
    
    public static IMarker addMarker(NamedElement elt, String description, int severity) {
        System.err.println(description);
		IResource onRsrc = getModelResource(elt);
        if (onRsrc == null) return null;
        try {
            IMarker marker = onRsrc.createMarker(IMarker.PROBLEM);
            if (marker.exists()) {
                marker.setAttribute(IMarker.MESSAGE, description);
                marker.setAttribute(IMarker.SEVERITY, severity);
                marker.setAttribute(IMarker.TRANSIENT, true);
                return marker;
            }
        } catch (CoreException e) {
            System.err.println("Marker does not exist."); //$NON-NLS-1$
        }
        return null;
    }

	public static List<EObject> findElements(EObject inObj, EClass inElementKind) {
		List<EObject> temp = new ArrayList<EObject>();
		for (Object curObj_o:inObj.eContents()) {
			EObject curObj = (EObject) curObj_o;
			if (inElementKind.isSuperTypeOf(curObj.eClass())) {
				temp.add((EObject) curObj);
			}
			temp.addAll(findElements((EObject) curObj_o, inElementKind));
		}
		return temp;
	}
	
	public static Classifier getOtherEnd(Association inAssoc, Classifier inThisEnd){
		Classifier target = null;
		EList members = inAssoc.getMemberEnds();
		if (members.size() >= 2) {
			Property prop1 = (Property) members.get(0);
			Property prop2 = (Property) members.get(1);
			Type type1 = prop1.getType();
			Type type2 = prop2.getType();
			if (type1 instanceof Classifier && type2 instanceof Classifier) {
				Classifier cl1 = (Classifier) type1;
				Classifier cl2 = (Classifier) type2;
				if (cl1.equals(inThisEnd) && !inAssoc.getOwnedEnds().contains(prop2)) {
					target = cl2;
				} else if (cl2.equals(inThisEnd) && !inAssoc.getOwnedEnds().contains(prop1)) {
					target = cl1;
				}
			}
		}
		return target;
	}
}
