package com.auchan.proto.patterns.custom;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.VisibilityKind;

import com.ibm.xtools.patterns.core.IParameterDescriptor;
import com.ibm.xtools.patterns.framework.AbstractPatternDefinition;
import com.ibm.xtools.patterns.framework.AbstractPatternInstance;

public class PatternUtil {
	
	public static Profile applyAuchanProfile(Model inModel) {
		Profile resProfile = Util.findProfile("com.auchan.proto.frontend", "ProtoAuchan");
		if (resProfile != null) {
			inModel.applyProfile(resProfile);
		}
		return resProfile;
	}
	
	public static void processEcranWithPopup(AbstractPatternInstance inPatternInstance) {
		Class ecran = null;
		Class popup = null;
		AbstractPatternDefinition patternDef = inPatternInstance.getOwningPatternDefinition();
		for (IParameterDescriptor pd : patternDef.getParameters()) {
			for (Object value : inPatternInstance.getParameterValues(pd)) {
				if (value instanceof Class) {
					Class cl = (Class) value;
					if (pd.getName().equals("Ecran")) {
						ecran = cl;
					} else if (pd.getName().equals("Popup")) {
						popup = cl;
					}
				}
			}
		}
		if (ecran != null && popup != null) {
			Association popupAssoc = null;
			List<EObject> assocs = Util.findElements(ecran.getModel(), UMLPackage.eINSTANCE.getAssociation());
			for (EObject curAssoc_o : assocs) {
				Association curAssoc = (Association) curAssoc_o;
				Classifier otherEnd = Util.getOtherEnd(curAssoc, ecran);
				if (otherEnd != null && otherEnd == popup) {
					popupAssoc = curAssoc;
					break;
				}
			}
			if (popupAssoc == null) {
				popupAssoc = ecran.createAssociation(true, AggregationKind.SHARED_LITERAL, popup.getName().toLowerCase(), 0, -1, popup, false, AggregationKind.NONE_LITERAL, ecran.getName().toLowerCase(), 1, -1);
				for (Object o : popupAssoc.getMemberEnds()) {
					Property prop = (Property) o;
					prop.setVisibility(VisibilityKind.PRIVATE_LITERAL);
				}
			}
			
			Stereotype st = popupAssoc.getAppliedStereotype("ProtoAuchan::popup");
			if (st == null) {
				st = popupAssoc.getApplicableStereotype("ProtoAuchan::popup");
				if (st != null) popupAssoc.applyStereotype(st);
			}
		}
	}

}
