package com.auchan.proto.patterns.custom;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.osgi.framework.Bundle;

import com.ibm.xtools.modeler.ui.UMLModeler;

public class Util {

	public static Profile findProfile(String inExtension, String inProfileName) {
		Profile resProfile = null;
		IExtension ext = findExtension("com.ibm.xtools.uml.msl.UMLProfiles", inExtension);
		if (ext != null) {
			IConfigurationElement ce = findConfigElement(ext, inProfileName);
			if (ce != null) {
				Bundle b = Platform.getBundle(inExtension);
				if (b != null) {
					String s = ce.getAttribute("path");
					int pos = s.lastIndexOf("/");
					if (pos > 0) s = s.substring(pos+1);
					// TODO Conversion of "pathmap://ARCHITECT_ASSISTANT_PROFILES/ArchitectAssistant.epx" to URL
					Enumeration e = b.findEntries("/", "*" + s, true);
					if (e != null) {
						while (e.hasMoreElements()) {
							URL u = (URL) e.nextElement();
							try {
								resProfile = UMLModeler.openProfile(FileLocator.resolve(u).getPath().substring(1));
							} catch (IOException e2) {
								// TODO Auto-generated catch block
								e2.printStackTrace();
							}
						}
					}
				}
			}
		}
		return resProfile;
	}
	
	public static IExtension findExtension(String inExtensionPointName, String inExtensionId) {
		IExtensionRegistry eR = Platform.getExtensionRegistry();
		IExtensionPoint ep = eR.getExtensionPoint(inExtensionPointName);
		IExtension resExt = null;
		if (ep != null) {
			resExt = ep.getExtension(inExtensionId);
			if (resExt == null) {
				for (IExtension e : ep.getExtensions()) {
					if (e.getNamespaceIdentifier().equals(inExtensionId)) {
						resExt = e;
						break;
					}
				}
			}
		}		
		return resExt;
	}

	public static IConfigurationElement findConfigElement(IExtension inExt, String inConfigElementName) {
		IConfigurationElement ce = null;
		if (inExt != null) {
			for (IConfigurationElement ice : inExt.getConfigurationElements()) {
				if (ice.getAttribute("name").equals(inConfigElementName)) {
					ce = ice;
					break;
				}
			}
		}			
		return ce;
	}
	
	public static List<EObject> findElements(EObject inObj, EClass inElementKind) {
		List<EObject> temp = new ArrayList<EObject>();
		for (Object curObj_o:inObj.eContents()) {
			EObject curObj = (EObject) curObj_o;
			if (inElementKind.isSuperTypeOf(curObj.eClass())) {
				temp.add((EObject) curObj);
			}
			temp.addAll(findElements((EObject) curObj_o, inElementKind));
		}
		return temp;
	}
	
	public static Classifier getOtherEnd(Association inAssoc, Classifier inThisEnd){
		Classifier target = null;
		EList members = inAssoc.getMemberEnds();
		if (members.size() >= 2) {
			Property prop1 = (Property) members.get(0);
			Property prop2 = (Property) members.get(1);
			Type type1 = prop1.getType();
			Type type2 = prop2.getType();
			if (type1 instanceof Classifier && type2 instanceof Classifier) {
				Classifier cl1 = (Classifier) type1;
				Classifier cl2 = (Classifier) type2;
				if (cl1.equals(inThisEnd)) {
					target = cl2;
				} else if (cl2.equals(inThisEnd)) {
					target = cl1;
				}
			}
		}
		return target;
	}
	
}
